<?php
/**
 * System Status plugin handler
 *
 * @package blesta
 * @subpackage blesta.plugins.system_status
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class SystemStatusPlugin extends Plugin
{
    public function __construct()
    {
        $this->loadConfig(dirname(__FILE__) . DS . 'config.json');

        Language::loadLang('system_status_plugin', null, dirname(__FILE__) . DS . 'language' . DS);
    }

    /**
     * Performs any necessary bootstraping actions
     *
     * @param int $plugin_id The ID of the plugin being installed
     */
    public function install($plugin_id)
    {
        $this->upgrade1_7_0($plugin_id, false);
    }

    /**
     * Performs migration of data from $current_version (the current installed version)
     * to the given file set version
     *
     * @param string $current_version The current installed version of this plugin
     * @param int $plugin_id The ID of the plugin being upgraded
     */
    public function upgrade($current_version, $plugin_id)
    {

        // Upgrade if possible
        if (version_compare($this->getVersion(), $current_version, '>')) {
            // Handle the upgrade, set errors using $this->Input->setErrors() if any errors encountered

            // Upgrade to 1.7.0
            if (version_compare($current_version, '1.7.0', '<')) {
                $this->upgrade1_7_0($plugin_id);
            }

            // Upgrade to 1.7.2
            if (version_compare($current_version, '1.7.2', '<')) {
                $this->upgrade1_7_2();
            }
        }
    }

    /**
     * Update to v1.7.0
     *
     * @param int $plugin_id The ID of the plugin being upgraded
     * @param bool $manage_acl Whether to allow/deny permissions using ACL
     */
    private function upgrade1_7_0($plugin_id, $manage_acl = true)
    {
        Loader::loadModels($this, ['Permissions']);
        Loader::loadComponents($this, ['Acl', 'Record', 'Input']);

        // Add new permissions for the system status widget
        $billing_group = $this->Permissions->getGroupByAlias('admin_billing');
        $billing_perm = [
            'plugin_id' => $plugin_id,
            'group_id' => $billing_group->id,
            'name' => Language::_('SystemStatusPlugin.name', true),
            'alias' => 'system_status.admin_main',
            'action' => 'billing'
        ];
        $this->Permissions->add($billing_perm);

        $dashboard_group = $this->Permissions->getGroupByAlias('admin_main');
        $dashboard_perm = [
            'plugin_id' => $plugin_id,
            'group_id' => $dashboard_group->id,
            'name' => Language::_('SystemStatusPlugin.name', true),
            'alias' => 'system_status.admin_main',
            'action' => '*'
        ];
        $this->Permissions->add($dashboard_perm);

        if (($errors = $this->Permissions->errors())) {
            $this->Input->setErrors($errors);
            return;
        }

        if ($manage_acl) {
            // Add access to all staff members for the system status widget
            $staff_groups = $this->Record->select(['staff_groups.id'])->
                from('staff_groups')->
                innerJoin('plugins', 'plugins.company_id', '=', 'staff_groups.company_id', false)->
                where('plugins.dir', '=', 'system_status')->
                group('staff_groups.id')->
                fetchAll();
            foreach ($staff_groups as $staff_group) {
                $this->Acl->deny('staff_group_' . $staff_group->id, 'system_status.admin_main', 'billing');
            }
        }
    }

    /**
     * Update to v1.7.2
     */
    private function upgrade1_7_2()
    {
        Loader::loadComponents($this, ['Acl', 'Record']);

        // Add access to all staff members for the system status widget
        $staff_groups = $this->Record->select(['staff_groups.id'])->
            from('staff_groups')->
            innerJoin('plugins', 'plugins.company_id', '=', 'staff_groups.company_id', false)->
            where('plugins.dir', '=', 'system_status')->
            group('staff_groups.id')->
            fetchAll();
        foreach ($staff_groups as $staff_group) {
            $this->Acl->removeAcl('staff_group_' . $staff_group->id, 'system_status.admin_main', '*');
            $this->Acl->allow('staff_group_' . $staff_group->id, 'system_status.admin_main', '*');
        }
    }

    /**
     * Performs any necessary cleanup actions
     *
     * @param int $plugin_id The ID of the plugin being uninstalled
     * @param bool $last_instance True if $plugin_id is the last instance across
     *  all companies for this plugin, false otherwise
     */
    public function uninstall($plugin_id, $last_instance)
    {
        Loader::loadModels($this, ['Permissions']);

        $biiling_permission = $this->Permissions->getByAlias('system_status.admin_main', $plugin_id, '*');
        if ($biiling_permission) {
            $this->Permissions->delete($biiling_permission->id);
        }

        $dashboard_permission = $this->Permissions->getByAlias('system_status.admin_main', $plugin_id, 'dashboard');
        if ($dashboard_permission) {
            $this->Permissions->delete($dashboard_permission->id);
        }
    }

    /**
     * Returns all actions to be configured for this widget
     * (invoked after install() or upgrade(), overwrites all existing actions)
     *
     * @return array A numerically indexed array containing:
     *  - action The action to register for
     *  - uri The URI to be invoked for the given action
     *  - name The name to represent the action (can be language definition)
     */
    public function getActions()
    {
        return [
            [
                'action' => 'widget_staff_home',
                'uri' => 'widget/system_status/admin_main/',
                'name' => 'SystemStatusPlugin.name'
            ],
            [
                'action' => 'widget_staff_billing',
                'uri' => 'widget/system_status/admin_main/billing',
                'name' => 'SystemStatusPlugin.name',
                'enabled' => 0
            ]
        ];
    }
}
